require 'set'
require 'uu/os/lang/constant_helper'

module UU
  module OS
    module Util

      # Module providing methods for lookup source script of given class.
      # @private
      module SourceLocator # :nodoc:

        # Finds source script (or multiple scripts) of given class. Method
        # internally uses native ruby function +source_location+ which is
        # available on +Method+ object. Therefore source can be found only
        # if class defines some own methods (either static or instance
        # methods). Since class might be reopened in multiple scripts, result
        # is returned as array containing all found source scripts.
        # @param [Class, String] clazz Class which source we are looking for
        # @return [Array<String>] List of source files (might be empty if class
        #    does not include any method or is part of stdlib).
        def find_source(clazz)
          SourceLocator.find_source(clazz)
        end

        # @see #find_source
        # @private
        def self.find_source(clazz) # :nodoc:
          sources = Set.new
          clazz = UU::OS::Lang::ConstantHelper.constant_get(clazz) if clazz.kind_of?(String)
          clazz.instance_methods(false).each do |method|
            method = clazz.instance_method(method)
            src = method.source_location
            sources << src[0] if !src.nil?
          end
          clazz.methods(false).each do |method|
            method = clazz.method(method)
            src = method.source_location
            sources << src[0] if !src.nil?
          end
          sources.to_a
        end

      end

    end
  end
end
